unit uSonyFontCLI;

interface

uses
  SysUtils, Classes,
  uSonyFont, uTrueType, uTrueTypeConst, uTrueTypeInternalStruct, uStringUtils;

type
  TSonyFontCLI = class
    private
      fFamily: TSonyFontFamily;
      fFamilySet: boolean;
      procedure SetFamily(Value: TSonyFontFamily);
    protected
      procedure ShowHeader; virtual;
      function ShowSyntax: integer; virtual;
      procedure ProcessParam(const Param: string); virtual;
      procedure UpdateFontFile(const FileName: string); virtual;
      procedure ShowFontFile(const FileName: string); virtual;
      property Family: TSonyFontFamily read fFamily write SetFamily;
      property FamilySet: boolean read fFamilySet;
    public
      constructor Create; virtual;
      destructor  Destroy; override;
      function    Execute: integer; virtual;
    end;

implementation

const
  VERSION = '0.01';

const
  ERR_ERROR = 255;
  ERR_SYNTAX = 254;
  ERR_ABORT = 253;
  ERR_OK = 0;

{ TSonyFontCLI }

constructor TSonyFontCLI.Create;
begin
  inherited Create;
  fFamilySet := False;
end;

destructor TSonyFontCLI.Destroy;
begin
  inherited;
end;

procedure TSonyFontCLI.SetFamily(Value: TSonyFontFamily);
begin
  fFamily := Value;
  fFamilySet := True;
end;

function TSonyFontCLI.Execute: integer;
var i: integer;
    s: string;
begin
  ShowHeader;
  if ParamCount = 0 then
    Result := ShowSyntax
  else
    begin
    Result := ERR_OK;
    for i := 1 to ParamCount do
      begin
      s := ParamStr(i);
      try
        if s[1] = '-' then
          ProcessParam(s)
        // For some strange reason, FileExists doesn't return true for files
        // extracted from image, even though the files open just fine.
        else if True or FileExists(s) then
          if FamilySet then
            UpdateFontFile(s)
          else
            ShowFontFile(s)
        else
          Raise EInOutError.CreateFmt('File "%s" not found.', [s]);
      except
        on EAbort do
          begin
          Result := ERR_ABORT;
          Break;
          end;
        on E: Exception do
          begin
          Writeln('ERROR ' + E.ClassName + ': ' + E.Message);
          Inc(Result);
          end;
        end;
      end;
    end;
end;

procedure TSonyFontCLI.ShowHeader;
begin
  Writeln('SonyFont v0.01                                                   (c) 2010 Pepak');
  Writeln('-------------------------------------------------------------------------------');
  Writeln('Update TTF files for use with Sony Readers.');
  Writeln('');
end;

function TSonyFontCLI.ShowSyntax: integer;
begin
  Writeln('Syntax:');
  Writeln('    SonyFont.exe <param> [<param>] [<param>] ...');
  Writeln('');
  Writeln('        <param> may be a filename (no wildcards) or one of the following:');
  Writeln('    -serif ......... Update the font as serif (e.g. Times New Roman)');
  Writeln('    -f               dtto');
  Writeln('    -sans-serif .... Update the font as sans-serif (e.g. Arial)');
  Writeln('    -s, -sans        dtto');
  Writeln('    -monospace ..... Update the font as monospace (e.g. Courier New)');
  Writeln('    -m, -mono        dtto');
  Writeln('    -alt-mono ...... Update the font as alternate monospace. This naming');
  Writeln('    -a, -amono       convention is almost certainly wrong, but it is used');
  Writeln('                     by PRS Customizer and some hacks (e.g. PRS Plus).');
  Writeln('    -view .......... Don''t update, just show font naming info');
  Writeln('    -v               dtto');
  Writeln('');
  Writeln('Example:');
  Writeln('    SonyFont.exe -f times.ttf -s arial.ttf -m cour.ttf -v tt0003_.ttf');
  Writeln('');
  Result := ERR_SYNTAX;
end;

procedure TSonyFontCLI.ProcessParam(const Param: string);
begin
  if (Param = '-f') or (Param = '-serif') then
    Family := sffSerif
  else if (Param = '-s') or (Param = '-sans-serif') or (Param = '-sans') then
    Family := sffSansSerif
  else if (Param = '-m') or (Param = '-monospace') or (Param = '-mono') then
    Family := sffMonospace
  else if (Param = '-a') or (Param = '-alt-mono') or (Param = '-amono') then
    Family := sffMonospaceAlt
  else if (Param = '-v') or (Param = '-view') then
    fFamilySet := False
  else if (Param = '-?') or (Param = '-h') or (Param = '-help') then
    begin
    ShowSyntax;
    Abort;
    end
  else
    Raise Exception.CreateFmt('Unknown argument "%s". Use -? or -h for list of supported arguments.', [Param]);
end;

procedure TSonyFontCLI.ShowFontFile(const FileName: string);
var FontFile: TTrueTypeFont;

  procedure Show(FontNameID: integer; const FontName: string; ShowMax: integer = MaxInt);
    var Names: TWideStringArray;
        i: integer;
    begin
      Write('  ' + FontName + ':');
      FontFile.GetAllFontNaming(FontNameID, Names);
      if Length(Names) > 0 then
        for i := 0 to Pred(Length(Names)) do
          begin
          if i = 0 then
            Write('   ')
          else
            Write('                 ');
          Writeln(WideToAnsi(Names[i]));
          Dec(ShowMax);
          if ShowMax <= 0 then
            Break;
          end
        else
          Writeln;
    end;

begin
  Writeln(FileName);
  FontFile := TTrueTypeFont.Create(nil);
  try
    FontFile.LoadFromFile(FileName);
    Show(TT_FONTNAMEID_FONTFAMILY,    'Font family');
    Show(TT_FONTNAMEID_FONTSUBFAMILY, 'Subfamily  ', 1);
    Show(TT_FONTNAMEID_FULLNAME,      'Full name  ');
  finally
    FontFile.Free;
    end;
  Writeln('');
end;

procedure TSonyFontCLI.UpdateFontFile(const FileName: string);
begin
  Write(FileName + '   ... ');
  try
    PrepareSonyFont(FileName, Family);
    Writeln('DONE');
  except
    Writeln;
    Raise;
    end;
end;

end.
